@@ -20,7 +20,7 @@ $sipc->connect(
 
 warn "client connected to server\n";
 
-$sipc->send("Hello server, gimme some data :-)\n", 1) or die $sipc->errstr($sipc->sock->errstr);
+$sipc->send_raw("Hello server, gimme some data :-)\n") or die $sipc->errstr($sipc->sock->errstr);
 my $answer = $sipc->read or die $sipc->errstr($sipc->sock->errstr);
 warn "server data: \n";
 warn Dumper($answer);
